package cz.sdc.wpstest;

import android.Manifest;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.net.wifi.ScanResult;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.os.Message;
import android.os.PowerManager;
import android.preference.PreferenceManager;
import android.provider.Settings;
import android.support.annotation.NonNull;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Gravity;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.TextView;
import android.widget.Toast;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

public class MainActivity extends AppCompatActivity {
    private static final int REQUEST_ACCESS_LOCATION = 10;
    private static final int TIMEOUT = 10000;

    private LocationManager gps;
    private WifiManager wifi;
    private BroadcastReceiver srr;
    private List<ScanResult> results;

    private ListView lv_networks;
    private ScanResultsAdapter adapter;
    private ScanResult selected;
    private ScanningDialog dlg_scan;
    private PinListDialog dlg_pinlist;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fab);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                startScan();
            }
        });

        lv_networks = findViewById(R.id.lv_networks);
        TextView tv_empty = findViewById(R.id.emptyList);
        lv_networks.setEmptyView(tv_empty);
        gps = (LocationManager)getApplicationContext().getSystemService(LOCATION_SERVICE);
        wifi = (WifiManager)getApplicationContext().getSystemService(WIFI_SERVICE);
        dlg_scan = new ScanningDialog(this);
        srr = new BroadcastReceiver() {

            @Override
            public void onReceive(Context context, Intent intent) {
                dlg_scan.dismiss();
                results = wifi.getScanResults();
                results = removeDuplicity(results);
                adapter = new ScanResultsAdapter(MainActivity.this, results);
                lv_networks.setAdapter(adapter);
            }
        };

        if(ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this, new String[] {Manifest.permission.ACCESS_COARSE_LOCATION}, REQUEST_ACCESS_LOCATION);
        }

        lv_networks.setOnItemClickListener(new AdapterView.OnItemClickListener() {

            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                selected = adapter.getItem(position);
                if(adapter.hasItemWpsActive(position)) {
                    AlertDialog.Builder dlg_choose = new AlertDialog.Builder(MainActivity.this);
                    dlg_choose.setMessage(getString(R.string.dlg_choose, selected.SSID));
                    ChooseDialogListener chooseDialogListener = new ChooseDialogListener();
                    dlg_choose.setPositiveButton(R.string.generated, chooseDialogListener);
                    dlg_choose.setNegativeButton(R.string.custom, chooseDialogListener);
                    dlg_choose.create().show();
                }
                else {
                    Snackbar.make(lv_networks, getString(R.string.wps_nactive), Snackbar.LENGTH_LONG).show();
                }
            }
        });
        SharedPreferences sprefs = PreferenceManager.getDefaultSharedPreferences(this);
        if(sprefs.getBoolean("firstrun", true)) {
            AlertDialog dlg_start = new AlertDialog.Builder(this)
                    .setMessage(R.string.dlg_start)
                    .setCancelable(false)
                    .setNeutralButton(android.R.string.ok, null)
                    .create();
            dlg_start.setCanceledOnTouchOutside(false);
            dlg_start.show();
            sprefs.edit().putBoolean("firstrun", false).apply();
        }
    }

    private List<ScanResult> removeDuplicity(List<ScanResult> results) {
        HashSet<String> set = new HashSet<>();
        Iterator<ScanResult> it = results.iterator();
        while(it.hasNext()) {
            ScanResult sr = it.next();
            if(!set.add(sr.SSID)) {
                it.remove();
            }
        }
        return results;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if(requestCode == REQUEST_ACCESS_LOCATION)
            if(grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                if(!gps.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
                    Intent gps_enable = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                    startActivity(gps_enable);
                }
            }
    }

    public void startScan() {
        if(!gps.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
            AlertDialog.Builder gpsEnableDialog = new AlertDialog.Builder(this)
                    .setMessage(R.string.gps_enable_msg)
                    .setCancelable(true)
                    .setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {

                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            Intent gps_enable = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                            startActivity(gps_enable);
                        }
                    }).setNegativeButton(android.R.string.no, null);
            gpsEnableDialog.create().show();
        }
        else {
            if(!wifi.isWifiEnabled()) {
                wifi.setWifiEnabled(true);
            }
            wifi.startScan();
            dlg_scan.show();
        }
    }

    public void pinLeaked() {
        AlertDialog.Builder dlg = new AlertDialog.Builder(this);
        dlg.setTitle(R.string.connected);
        dlg.setMessage(R.string.succeeded);
        dlg.setNeutralButton(android.R.string.ok, null);
        dlg.create().show();
    }

    public void failed() {
        AlertDialog.Builder dlg_failed = new AlertDialog.Builder(this);
        dlg_failed.setMessage(getString(R.string.failed));
        dlg_failed.setNeutralButton(android.R.string.ok, null);
        dlg_failed.create().show();
    }

    @Override
    protected void onResume() {
        super.onResume();
        registerReceiver(srr, new IntentFilter(WifiManager.SCAN_RESULTS_AVAILABLE_ACTION));
    }

    @Override
    protected void onPause() {
        super.onPause();
        unregisterReceiver(srr);
    }

    private class ChooseDialogListener implements DialogInterface.OnClickListener {

        @Override
        public void onClick(DialogInterface dialog, int which) {
            if(which == DialogInterface.BUTTON_POSITIVE) {
                dlg_pinlist = new PinListDialog(MainActivity.this, selected);
                PinListDialogListener pinListDialogListener = new PinListDialogListener();
                dlg_pinlist.setButton(DialogInterface.BUTTON_POSITIVE, getString(R.string.try_selected), pinListDialogListener);
                dlg_pinlist.setButton(DialogInterface.BUTTON_NEUTRAL, getString(R.string.try_all), pinListDialogListener);
                dlg_pinlist.show();
            }
            if(which == DialogInterface.BUTTON_NEGATIVE) {
                final CustomPinDialog customPinDialog = new CustomPinDialog(MainActivity.this);
                customPinDialog.setTitle(String.format("%s - %s", selected.SSID, getString(R.string.custom)));
                customPinDialog.setButton(DialogInterface.BUTTON_POSITIVE, getString(R.string.try_selected), (Message)null);
                customPinDialog.show();
                customPinDialog.getButton(DialogInterface.BUTTON_POSITIVE).setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        if(customPinDialog.isPinAcceptable()) {
                            customPinDialog.dismiss();
                            WpsConnector connector = new WpsConnector(MainActivity.this, selected.SSID, 1);
                            connector.setConfig(selected.BSSID, customPinDialog.getPin());
                            connector.connect(new WifiManager.WpsCallback() {
                                @Override
                                public void onStarted(String pin) {

                                }

                                @Override
                                public void onSucceeded() {
                                    pinLeaked();
                                }

                                @Override
                                public void onFailed(int reason) {
                                    wifi.cancelWps(null);
                                    failed();
                                }
                            });
                        } else {
                            Toast t = Toast.makeText(MainActivity.this, R.string.tooshort, Toast.LENGTH_SHORT);
                            t.setGravity(Gravity.CENTER, 0, 0);
                            t.show();
                        }
                    }
                });
            }
        }
    }

    private class PinListDialogListener implements DialogInterface.OnClickListener {

        @Override
        public void onClick(DialogInterface dialog, int which) {
            if(which == DialogInterface.BUTTON_POSITIVE) {
                final String pin = ((RadioButton) dlg_pinlist.rg.findViewById(dlg_pinlist.rg.getCheckedRadioButtonId())).getText().toString();
                final WpsConnector connector = new WpsConnector(MainActivity.this, selected.SSID, 1);
                connector.setConfig(selected.BSSID, pin);
                connector.connect(new WifiManager.WpsCallback() {
                    Timer t;
                    TimerTask tt;
                    @Override
                    public void onStarted(String pin) {
                        t = new Timer();
                        tt = new TimerTask() {
                            @Override
                            public void run() {
                                connector.finish();
                                wifi.cancelWps(null);
                                runOnUiThread(new Runnable() {
                                    @Override
                                    public void run() {
                                        failed();
                                    }
                                });
                            }
                        };
                        t.schedule(tt, TIMEOUT);
                    }

                    @Override
                    public void onSucceeded() {
                        t.cancel();
                        pinLeaked();
                    }

                    @Override
                    public void onFailed(int reason) {
                        connector.finish();
                        failed();
                    }
                });
            }

            if(which == DialogInterface.BUTTON_NEUTRAL) {
                final String[] pins = dlg_pinlist.pins;
                final WpsConnector connector = new WpsConnector(MainActivity.this, selected.SSID, pins.length);
                connector.setConfig(selected.BSSID, pins[0]);
                connector.connect(new WifiManager.WpsCallback() {
                    int i = 0;
                    PowerManager.WakeLock wl;
                    Timer t;
                    TimerTask tt;

                    @Override
                    public void onStarted(String pin) {
                        connector.dialog.getButton(DialogInterface.BUTTON_NEUTRAL).setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                wl.release();
                                connector.finish();
                                i = pins.length - 1;
                            }
                        });
                        PowerManager powerManager = (PowerManager) getSystemService(POWER_SERVICE);
                        wl = powerManager.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, "WPSWakeLock");
                        wl.acquire(10*60*1000L /*10 minutes*/);
                        t = new Timer();
                        tt = new TimerTask() {
                            @Override
                            public void run() {
                                runOnUiThread(new Runnable() {
                                    @Override
                                    public void run() {
                                        onFailed(223);
                                    }
                                });
                            }
                        };
                        t.schedule(tt, TIMEOUT);
                    }

                    @Override
                    public void onSucceeded() {
                        wl.release();
                        pinLeaked();
                        t.cancel();
                    }

                    @Override
                    public void onFailed(int reason) {
                        if (++i < pins.length) {
                            t.cancel();
                            wifi.cancelWps(null);
                            connector.setConfig(selected.BSSID, pins[i]);
                            connector.connect(this);
                            t = new Timer();
                            tt = new TimerTask() {
                                @Override
                                public void run() {
                                    runOnUiThread(new Runnable() {
                                        @Override
                                        public void run() {
                                            onFailed(223);
                                        }
                                    });
                                }
                            };
                            t.schedule(tt, TIMEOUT);
                        } else {
                            wl.release();
                            connector.finish();
                            runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    failed();
                                }
                            });
                        }
                    }
                });
            }
        }
    }
}
