package cz.sdc.wpstest;


import android.util.Log;

import java.util.ArrayList;

public class PinGenerator {
    private static PinGenerator instance;

    private PinGenerator() { }

    class PinList extends ArrayList<String> {
        public boolean add(long n) {
            String s = pinChecksum(n);
            s = zeroFill(s, 8);
            return super.add(s);
        }
    }

    public static PinGenerator getInstance() {
        if(instance == null)
            instance = new PinGenerator();
        return instance;
    }

    public String[] generatePins(String bssid) {
        final long mac = Long.parseLong(bssid.replace(":", ""), 16);
        PinList pinlist = new PinList();
        pinlist.add("12345670");
        pinlist.add(mac & 0xFFFFFF);
        pinlist.add(mac & 0xFFFFFFF);
        pinlist.add(mac % 0x100000000L);
        pinlist.add(mac % 0x1000000000L);
        pinlist.add(mac % 0x10000000000L);
        pinlist.add(mac % 0x100000000000L);
        pinlist.add(mac);

        long m = mac & 0xFFFFFF;
        String ms = zeroFill(Long.toString(m), 6);
        ms = Long.toString(Long.parseLong(ms), 16);
        ms = ms.substring(4, 6) + ms.substring(2,4) + ms.substring(0,2);
        pinlist.add(Long.parseLong(ms, 16));

        m = mac % 0x100000000L;
        ms = zeroFill(Long.toString(m), 8);
        ms = zeroFill(Long.toString(Long.parseLong(ms), 16), 8);
        ms = ms.substring(6, 8) + ms.substring(4,6) + ms.substring(2,4) + ms.substring(0,2);
        pinlist.add(Long.parseLong(ms, 16));

        m = mac;
        ms = zeroFill(Long.toString(m), 12);
        ms = zeroFill(Long.toString(Long.parseLong(ms), 16), 12);//8
        ms = ms.substring(10,12) + ms.substring(8,10) + ms.substring(6,8) + ms.substring(4,6)
                + ms.substring(2,4)
                + ms.substring(0,2);
        pinlist.add(Long.parseLong(ms, 16));

        m = mac & 0xFFFFFF;
        ms = zeroFill(Long.toString(m), 6);
        ms = zeroFill(Long.toString(Long.parseLong(ms), 16), 6);//8
        pinlist.add(Long.parseLong(new StringBuilder(ms).reverse().toString(), 16));

        m = mac % 0x100000000L;
        ms = zeroFill(Long.toString(m), 8);
        ms = zeroFill(Long.toString(Long.parseLong(ms), 16), 8);
        pinlist.add(Long.parseLong(new StringBuilder(ms).reverse().toString(), 16));

        ms = zeroFill(Long.toString(mac), 12);
        ms = zeroFill(Long.toString(Long.parseLong(ms), 16), 12);
        pinlist.add(Long.parseLong(new StringBuilder(ms).reverse().toString(), 16));

        m = mac & 0xFFFFFF;
        ms = zeroFill(Long.toString(m), 6);
        ms = zeroFill(Long.toString(Long.parseLong(ms), 2), 24);
        pinlist.add(Long.parseLong(new StringBuilder(ms).reverse().toString(), 2));

        m = mac % 0x100000000L;
        ms = zeroFill(Long.toString(m), 8);
        ms = zeroFill(Long.toString(Long.parseLong(ms), 2), 32);
        pinlist.add(Long.parseLong(new StringBuilder(ms).reverse().toString(), 2));

        ms = zeroFill(Long.toString(mac), 12);
        ms = zeroFill(Long.toString(Long.parseLong(ms), 2), 48);
        pinlist.add(Long.parseLong(new StringBuilder(ms).reverse().toString(), 2));

        pinlist.add(algoDLink(mac));
        pinlist.add(algoDLink(mac + 1));
        pinlist.add(algoDSL_MAC_SN(mac, null, new int[]{66, 129, 209, 10, 24, 3, 39}));
        pinlist.add(algoDSL_MAC_SN(mac, Long.toString(mac & 0xFFFF, 10), new int[]{129, 65, 6, 10, 136, 80, 33}));
        pinlist.add(algoDSL_MAC_SN(mac - 2, null, new int[]{129, 65, 6, 10, 136, 80, 33}));
        pinlist.add(algoAsus(mac));
        pinlist.add(algoAirocon(mac));
        pinlist.add(~(mac & 0xFFFFFF) & 0xFFFFFF);
        pinlist.add((mac & 0xFFFFFF) * 2);
        pinlist.add((mac & 0xFFFFFF) * 3);

        ms = zeroFill(Long.toString(mac, 16), 12);
        int oui = Integer.parseInt(ms.substring(0, 6), 16);
        int nic = Integer.parseInt(ms.substring(6, 12), 16);
        pinlist.add((oui + nic) % 0x1000000);

        if (nic < oui) {
            pinlist.add(oui - nic);
        } else {
            pinlist.add((oui + 0x1000000 - nic) & 0xFFFFFF);
        }
        //pinlist.add(oui ^ nic);

        return pinlist.toArray(new String[pinlist.size()]);
    }

    private String pinChecksum(long pin) {
        pin %= 10000000;
        int a = 0;
        long t = pin;
        while (t > 0) {
            a += 3 * (t % 10);
            t = t / 10;
            a += t % 10;
            t /= 10;
        }
        return String.valueOf((pin * 10) + ((10 - (a % 10)) % 10));
    }

    private String zeroFill(String number, int width) {
        width -= number.length();
        if (width > 0) {
            StringBuilder sb = new StringBuilder();
            for(int i = 0; i < width; i++)
                sb.append("0");
            sb.append(number);
            return sb.toString();
        }
        return number;
    }

    private long algoDLink(long mac) {
        mac &= 0xFFFFFF;
        mac ^= 0x55AA55;
        mac ^= ((mac & 0xF) << 4)
                | ((mac & 0xF) << 8)
                | ((mac & 0xF) << 12)
                | ((mac & 0xF) << 16)
                | ((mac & 0xF) << 20);
        mac %= 10000000;
        if (mac < 1000000) mac += ((mac % 9) * 1000000) + 1000000;
        return mac;
    }

    private long algoDSL_MAC_SN(long mac, String ser, int[] bx) {
	/*
		universal algorithm, returns pin derived from MAC and S/N
		used by: Belkin, DSL-EasyBox, Arcadyan, and possibly other manufacturers

		reverse-engineered by Stas'M
	*/
        if (ser == null)
            ser = "";
        if (ser.length() < 4)
            ser = zeroFill(ser, 4);
        if (ser.length() > 4)
            ser = ser.substring(ser.length() - 4, ser.length());
        int[] sn = new int[ser.length()];
        for (int i = 0; i < ser.length(); i++) {
            int x = Integer.parseInt(String.valueOf(ser.charAt(i)), 16);
            //if (isNaN(x)) x = 0;
            sn[i] = x;
        }
        long[] nic = new long[]{(mac & 0xFFFF) >> 12, (mac & 0xFFF) >> 8, (mac & 0xFF) >> 4, mac & 0xF};

	/*
		init vector:
		bk1  - k1 sum composite bits (0~255)
		bk2  - k2 sum composite bits (0~255)
		bx[] - xor composite bits (array of 0~255)
		k1   - k1 init nibble (0~15)
		k2   - k2 init nibble (0~15)
		pin  - pin init value (0~9999999)
		xor  - xor init nibble (0~15)
		sub  - substraction mode (0 - no sub, 1 - substraction, 2 - addition)
		sk   - substraction multiple coefficient (0 - skv value, 1 - k1, 2 - k2)
		skv  - substraction multiple value
	*/
	int[] init = new int[] {60, 195, 0, 0, 0, 0, 0, 0, 0};

        int k1 = init[2] & 0xF;
        int i = 0;
        while (init[0] > 0)
        {
            // 0      1      2      3      4     5     6     7
            // nic[0] nic[1] nic[2] nic[3] sn[0] sn[1] sn[2] sn[3]
            if ((init[0] & 1) > 0)
            {
                k1 += (i < 4 ? nic[i] : sn[i - 4]);
                k1 &= 0xF;
            }
            init[0] = init[0] >> 1;
            i++;
        }
        int k2 = init[3] & 0xF;
        i = 0;
        while (init[1] > 0)
        {
            // 0      1      2      3      4     5     6     7
            // nic[0] nic[1] nic[2] nic[3] sn[0] sn[1] sn[2] sn[3]
            if ((init[1] & 1) > 0)
            {
                k2 += (i < 4 ? nic[i] : sn[i - 4]);
                k2 &= 0xF;
            }
            init[1] = init[1] >> 1;
            i++;
        }

        int pin = init[4];
        for (int j = 0; j < bx.length; j++)
        {
            int xor = init[5] & 0xF;
            i = 0;
            while (bx[j] > 0)
            {
                // 0   1   2       3       4       5      6      7
                // k1, k2, nic[1], nic[2], nic[3], sn[1], sn[2], sn[3]
                if ((bx[j] & 1) > 0)
                {
                    xor ^= (i > 4 ? sn[i - 4] : (i > 1 ? nic[i - 1] : (i > 0 ? k2 : k1)));
                }
                bx[j] = bx[j] >> 1;
                i++;
            }
            pin <<= 4;
            pin |= xor;
        }
        switch (init[6]) {
            case 0: // by default
                return pin % 10000000;
            case 1: // spoiled from Belkin's reversed algo
                return (long)((pin % 10000000) - (Math.floor(pin / 10000000) * (init[7] > 1 ? k2 : (init[7] > 0 ? k1 : init[8]))));
            default:
                return (long)((pin % 10000000) + (Math.floor(pin / 10000000) * (init[7] > 1 ? k2 : (init[7] > 0 ? k1 : init[8]))));
        }
    }

    private long algoAsus(long mac) {
        String ms = zeroFill(Long.toString(mac, 16), 12);
        Log.d("DEBUG", ms);
        int[] b = {Integer.parseInt(ms.substring(0, 2), 16),
                Integer.parseInt(ms.substring(2, 4), 16),
                Integer.parseInt(ms.substring(4, 6), 16),
                Integer.parseInt(ms.substring(6, 8), 16),
                Integer.parseInt(ms.substring(8, 10), 16),
                Integer.parseInt(ms.substring(10, 12), 16)
        };
        String pin = "";
        for (int i = 0; i < 7; i++)
        {
            pin += String.valueOf((b[i % 6] + b[5]) % (10 - ((i + b[1] + b[2] + b[3] + b[4] + b[5]) % 7)));
        }
        return Long.parseLong(pin, 10);
    }

    private long algoAirocon(long mac) {
        String ms = zeroFill(Long.toString(mac, 16), 12);
        int[] b = {Integer.parseInt(ms.substring(0, 2), 16),
                Integer.parseInt(ms.substring(2, 4), 16),
                Integer.parseInt(ms.substring(4, 6), 16),
                Integer.parseInt(ms.substring(6, 8), 16),
                Integer.parseInt(ms.substring(8, 10), 16),
                Integer.parseInt(ms.substring(10, 12), 16)
        };
        return ((b[0] + b[1]) % 10)
                + (((b[5] + b[0]) % 10) * 10)
                + (((b[4] + b[5]) % 10) * 100)
                + (((b[3] + b[4]) % 10) * 1000)
                + (((b[2] + b[3]) % 10) * 10000)
                + (((b[1] + b[2]) % 10) * 100000)
                + (((b[0] + b[1]) % 10) * 1000000);
    }
}
