package cz.sdc.wpstest;

import android.net.wifi.ScanResult;
import android.net.wifi.WifiManager;
import android.support.v7.app.AppCompatActivity;
import android.text.Html;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;


public class ScanResultsAdapter extends BaseAdapter {
    private AppCompatActivity activity;
    private List<ScanResult> data;

    public ScanResultsAdapter(AppCompatActivity activity, List<ScanResult> data) {
        this.activity = activity;
        this.data = data;
        Collections.sort(this.data, new Comparator<ScanResult>() {

            @Override
            public int compare(ScanResult result1, ScanResult result2) {
                Integer level1 = WifiManager.calculateSignalLevel(result1.level, 100);
                Integer level2 = WifiManager.calculateSignalLevel(result2.level, 100);
                return level2.compareTo(level1);
            }
        });

    }

    @Override
    public int getCount() {
        return data.size();
    }

    @Override
    public ScanResult getItem(int i) {
        return data.get(i);
    }

    @Override
    public long getItemId(int i) {
        return i;
    }

    public boolean hasItemWpsActive(int i) {
        return data.get(i).capabilities.toLowerCase().contains("wps");
    }

    @Override
    public View getView(int i, View view, ViewGroup viewGroup) {
        if(view == null) {
            view = LayoutInflater.from(activity).inflate(R.layout.list_item, viewGroup, false);
        } else return view;
        ScanResult item = getItem(i);
        ImageView iv_signal = view.findViewById(R.id.iv_signal);
        TextView tv_ssid = view.findViewById(R.id.tv_ssid);
        TextView tv_bssid = view.findViewById(R.id.tv_bssid);
        TextView tv_vendor = view.findViewById(R.id.tv_vendor);
        TextView tv_security = view.findViewById(R.id.tv_security);
        String image_res_name = String.format("ic_signal_wifi_%d_bar%s", WifiManager.calculateSignalLevel(item.level, 5), hasItemWpsActive(i) ? "" : "_lock");
        int resId = activity.getResources().getIdentifier(image_res_name, "drawable", activity.getPackageName());
        iv_signal.setImageResource(resId);
        tv_ssid.setText(item.SSID);
        new VendorTask(activity, tv_vendor).execute(item.BSSID);
        tv_bssid.setText(item.BSSID.toUpperCase());
        tv_security.setText(Html.fromHtml("<small><small>" + item.capabilities.substring(1).split("]")[0] + "</small></small>"));
        tv_vendor.setText("");
        return view;
    }
}
